/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.apps.manifests.json;

import java.util.HashMap;
import java.util.Map;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.ajax.requesthandler.AJAXActionService;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.ajax.requesthandler.DispatcherNotes;
import com.openexchange.apps.manifests.ManifestContributor;
import com.openexchange.apps.manifests.json.osgi.ServerConfigServicesLookup;
import com.openexchange.capabilities.Capability;
import com.openexchange.capabilities.CapabilityService;
import com.openexchange.capabilities.CapabilitySet;
import com.openexchange.exception.OXException;
import com.openexchange.server.ServiceLookup;
import com.openexchange.tools.servlet.AjaxExceptionCodes;
import com.openexchange.tools.session.ServerSession;

/**
 * {@link AllAction}
 *
 * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
 */
@DispatcherNotes(noSession = true)
public class AllAction implements AJAXActionService {

    private final JSONArray manifests;
    private final ServiceLookup services;
    private final ServerConfigServicesLookup registry;

    public AllAction(ServiceLookup services, JSONArray manifests, ServerConfigServicesLookup registry) {
        super();
        this.manifests = manifests;
        this.services = services;
        this.registry = registry;
    }

    @Override
    public AJAXRequestResult perform(AJAXRequestData requestData, ServerSession session) throws OXException {
        return new AJAXRequestResult(getManifests(session, manifests, services, registry), "json");
    }

    public static JSONArray getManifests(ServerSession session, JSONArray manifests, ServiceLookup services, ServerConfigServicesLookup registry) throws OXException {
        manifests = new JSONArray(manifests);

        JSONArray result = new JSONArray();
        for(ManifestContributor contributors: registry.getContributors()) {
            try {
                JSONArray additionalManifests = contributors.getAdditionalManifests(session);
                if (additionalManifests != null) {
                    for(int i = 0, size = additionalManifests.length(); i < size; i++) {
                        manifests.put(additionalManifests.get(i));
                    }
                }
            } catch (OXException x) {
                // TODO: Logging
                x.printStackTrace();
            } catch (JSONException x) {
                // TODO: Logging
                x.printStackTrace();
            }
        }

        try {
            if (session.isAnonymous()) {
                // Deliver no apps and only plugins with the namespace 'signin'

                for (int i = 0, size = manifests.length(); i < size; i++) {
                    JSONObject definition = manifests.getJSONObject(i);
                    result.put(new JSONObject(definition));
                }

            } else {
                CapabilitySet capabilities = services.getService(CapabilityService.class).getCapabilities(session.getUserId(), session.getContextId(), true, true);

                Map<String, Capability> capMap = new HashMap<String, Capability>(capabilities.size());
                for (Capability capability : capabilities) {
                    capMap.put(capability.getId(), capability);
                }

                for (int i = 0, size = manifests.length(); i < size; i++) {
                    JSONObject definition = manifests.getJSONObject(i);
                    result.put(new JSONObject(definition));
                }
            }
        } catch (JSONException x) {
            throw AjaxExceptionCodes.JSON_ERROR.create(x.getMessage(), x);
        }
        return result;
    }
}
